/* 
 * kmolwidget.cpp 
 *
 * Copyright (C) 2000 Tomislav Gountchev <tomi@idiom.com>
 */

#include <kapp.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kpushbutton.h>

#include <qlineedit.h>
#include <qlayout.h>
#include <qmultilinedit.h>
#include <qlabel.h>
#include "kmolwidget.h"



/**
 * This class is the main view for KMol. It handles user input, passes it to KMOLCALC and
 * displays the result.
 *
 * @short Main view
 * @author Tomislav Gountchev <tomi@idiom.com>
 */
KMolWidget::KMolWidget (QWidget *parent, const char* name) 
    : QWidget (parent) {

        QGridLayout *grid = new QGridLayout ( this, 7, 2, 5, 5);

        //accepts text input until return is pressed 
        formula = new QLineEdit (this);
        formula->setFocus();
        formula->setFont(KGlobalSettings::fixedFont());
        connect(formula, SIGNAL(returnPressed()), this, SLOT(calc()));
        grid->addMultiCellWidget( formula , 1 ,1,0,1 );
        
        QLabel* formula_l = new QLabel (i18n("Formula"), this, "formula");
        grid->addWidget( formula_l , 0 , 0 );

        //mol weight output field
        result = new QLabel (this);
        result->setFrameStyle (QFrame::Panel | QFrame::Sunken);
        result->setLineWidth (2);
        result->setFont(KGlobalSettings::fixedFont());
        result->setBackgroundMode(PaletteBase);
        grid->addMultiCellWidget( result, 4, 4 ,0 ,1 );

        QLabel* result_l = new QLabel (i18n("Mw"), this, "result");
        grid->addWidget( result_l, 3 , 0 );

        //calculate button
        KPushButton* calculate = new KPushButton (i18n("Calculate"), this, "calculate");
        calculate->setDefault(TRUE);
        connect(calculate, SIGNAL(clicked()), this, SLOT(calc()));
        connect(formula, SIGNAL(returnPressed()), calculate, SLOT(animateClick()));
        grid->addWidget(calculate , 2, 0 );

        //clears FORMULA and RESULT fields
        KPushButton* clear_fields = new KPushButton (i18n("Clear"), this, "clear");
        connect(clear_fields, SIGNAL(clicked()), this, SLOT(clear()));
        grid->addWidget(clear_fields, 2,1 );

        //E.A. display field
        anal_display = new QMultiLineEdit (this, "display");
        anal_display->setReadOnly(1);
        anal_display->setFont(KGlobalSettings::fixedFont());
        anal_display->setFocusPolicy(QWidget::NoFocus);
        grid->addMultiCellWidget(anal_display, 6, 6, 0, 1 );

        QLabel* anal_display_l = new QLabel (i18n("Elemental Composition (%)"), this, "displaylabel");
        anal_display_l->setFont(KGlobalSettings::generalFont());
        grid->addMultiCellWidget(anal_display_l , 5 , 5 , 0 , 1 );

        kmolcalc = new KMolCalc;
}

KMolWidget::~KMolWidget() {
  delete kmolcalc;
}


/**  
 * Make a call to KMOLCALC to parse the string in FORMULA, if any, and get Mw and EA back.
 */
void KMolWidget::calc() {
  QString compound(formula->text());
  if (compound.isEmpty()) {
    clear();
    return;
  }
  QString errors(kmolcalc->readFormula(compound));
  QString mw, ea;
  double weight = kmolcalc->getWeight();
  if (errors == "OK") {
    mw.setNum(weight);
    ea = kmolcalc->getEmpFormula() + " :\n" + kmolcalc->getEA();
  } else {
    mw = "???";
    ea = i18n("ERROR: \n") + errors + "\n";
  }
  result->setText(mw);
  anal_display->setText(ea);
}

/**
 * Call the built in editor to edit the definitions file.
 */
void KMolWidget::callEditor() {
  KMolEdit* kmoledit = new KMolEdit(0, "kmoledit", kmolcalc);
  kmoledit->exec();
  delete kmoledit;
}


/** 
 * Clear all text entry / result fields.
 */
void KMolWidget::clear() {
  formula->clear();
  result->clear();
  anal_display->clear();
}


